from xml.sax.saxutils import escape,quoteattr
import re,pdb

quippleNamespace='http://www.thibault.org/Quipple'
jquoteNamespace="http://jducoeur.org/jquote/"
htmlNamespace='http://www.w3.org/1999/xhtml'

notWordRegexp=re.compile('\W')

class BodyText(object):
    def __init__(self,s):
        self.s=s

    def toXML(self):
        return escape(self.s)

    def __iter__(self):
        return iter(filter(None,notWordRegexp.split(self.s)))

    def flatten(self,emphasisLevel=0):
        return self.s

    def __add__(self,other):
        return BodyText(self.s+other.s)

    def replace(self,*args):
        return BodyText(self.s.replace(*args))

    def __str__(self):
        return self.s

class BodyTag(object):
    def __init__(self,tag,children=None):
        self.tag=tag
        self.children=[]
        for child in children:
            self.addChild(child)

    def addChild(self,child):
        if isinstance(child,str) or isinstance(child,unicode):
            child=BodyText(child)
        if (isinstance(child,BodyText)
            and self.children
            and isinstance(self.children[-1],BodyText)):
            self.children[-1]=BodyText(self.children[-1]+child)
        else:
            self.children.append(child)

    def toXML(self):
        res=u"<html:%s>" % (self.tag,)
        for child in self.children:
            res+=unicode(child.toXML())
        res+=u"</html:%s>" % (self.tag,)
        return res

    def __contains__(self,word):
        for c in self.children:
            if word in c:
                return True
        return False

    def __iter__(self):
        for c in self.children:
            for word in c:
                yield word

    def flatten(self,emphasisLevel=0):
        emphasisDelta=1 if self.tag=='em' else 0
        emphasisLevel+=emphasisDelta
        text=''.join(map(lambda c: str(c.flatten(emphasisLevel)),
                         self.children))
        if emphasisDelta:
            decor=['','*','_'][emphasisLevel]
            return decor+text+decor

        if self.tag=='p':
            return text+'\n'

        return text

def normalizeBody(body):
    def normalizeBodyPart(part):
        if isinstance(part,str) or isinstance(part,unicode):
            return BodyText(part)
        if isinstance(part,BodyTag) or isinstance(part,BodyText):
            return part
        raise ValueError('Body must be a string, list, BodyTag, or BodyText, not %s' % str(type(part)))

    if isinstance(body,list):
        return list(map(normalizeBodyPart,body))
    else:
        return [normalizeBodyPart(body)]

def mkId():
    mkId.next+=1
    return 'q%d' % mkId.next

mkId.next=0

class Quip:
    def __init__(self,body=None,cite=None,tags=None):
        self.body=normalizeBody(body)
        self.cite=cite
        self.tags=[]
        for tag in tags or []:
            self.addTag(tag)

    def setCite(self,url):
        self.cite=url

    def addTag(self,tag):
        self.tags.append(tag)

    def toText(self):
        pass

    def toXML(self,prefix=''):
        res="%s<quote>\n" % prefix

        res+="%s  <text>" % prefix
        for b in self.body:
            res+=escape(b.flatten().strip())
        res+="</text>\n"

        if self.cite:
            res+="%s  <attribution>%s</attribution>\n" % (prefix,escape(self.cite))
        if self.tags:
            res+="%s  <categories>\n" % prefix
            for tag in self.tags:
                if isinstance(tag,tuple):
                    tag=tag[0]
                res+="%s    <category>%s</category>\n" % (prefix,escape(tag))
            res+="%s  </categories>\n" % prefix

        res+="%s</quote>\n" % prefix
        return res

    def __contains__(self,word):
        for b in self.body:
            if word in b:
                return True
        return False

    def __iter__(self):
        for b in self.body:
            for word in b:
                yield word

    def containsTag(self,tag):
        return tag in self.tags

    def flatten(self):
        return ''.join(map(lambda b: b.flatten(),self.body)).strip()

def quipsToXML(quips,src=None):
    res="""<?xml version="1.0" encoding="UTF-8"?>

<quoteset xmlns=%s>
""" % quoteattr(jquoteNamespace)
    quipXML=''
    allTags=set()
    for quip in quips:
        if quip.tags:
            allTags.update(quip.tags)
        quipXML+=quip.toXML('    ')
    if allTags:
        res+="  <categories>\n"
        for tag in allTags:
            res+="    <category>%s</category>\n" % escape(tag)
        res+="  </categories>\n"
    res+="  <quotes>\n%s  </quotes>\n" % quipXML
    res+="</quoteset>\n"
    return res
