import quipple,math,random,urllib2,pdb

bodyWeight=1
tagWeight=1
maxRepeats=3

rand=random.Random()

def escapeCGI(value):
    return urllib2.quote(str(value))

def take(max,xs):
    if max is None:
        return xs
    def g():
        i=0
        for x in xs:
            if i>=max:
                return
            yield x
            i+=1
    return g()

def nontrivial(word):
    return not (len(word)<3 or word in nontrivial.trivialWords)

nontrivial.trivialWords=set(['the','for'])

class Query(object):
    def __init__(self,
                 words=None,max=None,shuffle=None,
                 tags=None):
        self.words=list(filter(nontrivial,words or []))
        self.tags=[]
        for tag in tags or []:
            self.addTag(tag)
        self.max=max
        self.shuffle=shuffle

    def toCGI(self):
        def params():
            for word in self.words:
                yield ('word',word)
            for tag in self.tags:
                yield ('tag',tag)
            if self.max is not None:
                yield ('max',self.max)
            if self.shuffle is not None:
                yield ('shuffle',self.shuffle)
        return '&'.join(map(lambda nameValue: (nameValue[0]
                                               +'='
                                               +escapeCGI(nameValue[1])
                                               ),
                            params()))

    def addTag(self,tag):
        self.tags.append(tag)

    def _matchesJustTags(self,quips):
        for quip in quips:
            for tag in self.tags:
                if quip.containsTag(tag):
                    yield quip

    def matchIgnoreTags(self,quip):
        score=0
        for word in self.words:
            word=word.lower()
            scoreboard=[]
            pos=1
            for w in quip:
                if w.lower()==word:
                    scoreboard.append(pos)
                if len(scoreboard)>=maxRepeats:
                    break
                pos+=1
            length=pos
            for (i,pos) in enumerate(scoreboard):
                score+=math.sqrt((length+1.0-pos)/length)/(i+1)
            score*=bodyWeight
            
            for tag in quip.tags:
                if word in tag.lower().split('_-.'):
                    score+=tagWeight
        return score

    def matches(self,quips):
        if self.tags:
            quips=self._matchesJustTags(quips)
        def g():
            for quip in quips:
                score=self.matchIgnoreTags(quip)
                if score:
                    yield (quip,score)
        if self.words:
            return g()
        else:
            return map(lambda q: (q,1),quips)

    def search(self,quips):
        def q(qscore):
            return qscore[0]
        def score(qscore):
            return qscore[1]
        found=map(q,sorted(self.matches(quips),key=score,reverse=True))
        if self.shuffle:
            found=set(found)
            if self.shuffle<0:
                shuffle=len(found)
            else:
                shuffle=min(self.shuffle,len(found))
            found=rand.sample(found,shuffle)
        return take(self.max,found)
